/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.p4java.extension.utility;

import java.util.Map;

import org.apache.log4j.Logger;

import com.perforce.p4java.exception.MessageSeverityCode;
import com.perforce.p4java.extension.command.ServerMessages;
import com.perforce.p4java.extension.exception.ServerAccessException;
import com.perforce.p4java.extension.exception.ServerConnectionException;
import com.perforce.p4java.extension.exception.ServerErrorException;
import com.perforce.p4java.impl.mapbased.rpc.msg.RpcMessage;

/**
 * @since 2012.1 TODO: convert command as subclass of Command, and get rid of
 *        me.
 */
public class CommandUtil {
    private static final Logger log = Logger.getLogger(CommandUtil.class);

    public static void checkError(Map<String, Object>[] maps)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	checkError(maps, null);
    }

    public static void checkError(Map<String, Object>[] maps, ErrorId[] errors)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {

	if (maps == null)
	    return;

	// check error and throw exception
	for (Map<String, Object> map : maps) {

	    for (int i = 0;; i++) {
		String code = "code" + i;
		if (map.containsKey(code)) {
		    String codeValue = (String) map.get(code);
		    String suffix = " serverCode=[" + codeValue + "]";

		    String message = ServerMessages.getServerMessage(map,
			    MessageSeverityCode.E_INFO);
		    boolean isAuthFail = ServerMessages.isAuthFail(message);

		    if (isAuthFail) {
			log.error(message + suffix);
			throw new ServerAccessException(message + suffix);
		    } else if (RpcMessage.getSeverity(codeValue) == MessageSeverityCode.E_WARN) {
			throw new ServerErrorException(message + suffix,
				codeValue);
		    } else if (RpcMessage.getSeverity(codeValue) >= MessageSeverityCode.E_WARN) {
			log.error(message + suffix);
			throw new ServerErrorException(message + suffix,
				codeValue);
		    } else if (isNotSeriousErrorButNeedReport(codeValue)) {
			log.error(message + suffix);
			throw new ServerErrorException(message + suffix,
				codeValue);
		    }
		    // loop through other errors too
		    if (errors != null)
			for (int eid = 0; eid < errors.length; eid++) {
			    if (errors[eid].match(codeValue))
				throw new ServerErrorException(
					message + suffix, codeValue);
			}
		} else
		    break;
	    }
	}
    }

    public static boolean isIgnorableWarnning(String code) {
	if (ErrorIds.MsgDm_ExFILE.match(code))
	    return true;
	return false;
    }

    public static boolean isNotSeriousErrorButNeedReport(String code) {
	if (ErrorIds.MsgDm_LockAlreadyOther.match(code))
	    return true;
	return false;
    }
}
